%% MARQUISE post series: Postulates of quantum mechanics
% Postulate 6
% Quentin Stern - Mar 25, 2025
clear

%% Showing the results as a video

omega0  = 2*pi*1;   % Larmor freq (rad.s-1)
dt      = 0.01;     % Time increments (s)
K       = 300;      % Number of increments (-)

% Time axis and empty vectors to store results 
% of numerical propatation
t       = dt*(0:K-1);
It      = zeros(3,K);

% Initial state
psi0    = [1; 1]/sqrt(2);

% Angular momentum operators
Ix      = [0 1; 1 0]/2;
Iy      = [0 -1i; +1i 0]/2;
Iz      = [+1 0; 0 -1]/2;

% Hamiltonian and propagator during dt
H0      = omega0*Iz;
U0      = expm(-1i*H0*dt);

% Initialization of the system state
psi = psi0;

% Loop for propagation
for k = 1:K
    % Expectation values at time (i-1)*dt
    It(1,k) = real(psi'*Ix*psi);
    It(2,k) = real(psi'*Iy*psi);
    It(3,k) = real(psi'*Iz*psi);

    % Propagation during dt
    psi = U0*psi;
end

% Creates a Bloch sphere
Figure1=BlochSpereFunc(FontSize);

% Plot options
color =[253 181 129]/254;
sha= 0.9;       % Shading for marker
sh=0.5;         % Shading for arrow
lw = 1.5;       % Linewdith
FontSize=12;    % Fontsize

% Initializes the plot area before generating the video
subplot(3,1,1)
c=get(gca,'ColorOrder');
px=animatedline('LineWidth',lw,'Color',c(1,:));
py=animatedline('LineWidth',lw,'Color',c(2,:));
pz=animatedline('LineWidth',lw,'Color',c(3,:));
yline([+1 -1]/2)
legend({'{\itI_x}','{\itI_y}','{\itI_z}'})
xlim([0 max(t)])
xlabel('Time (s)'), ylabel('<{\itI_\mu}> (-)')
yticks((-1:+1)/2), yticklabels({'-1/2','0','+1/2'})
set(gca,'FontSize',12)

% Initializes the plot area before generating the video
subplot(3,2,[3 5])
angle=2*pi*linspace(0,1,100);
plot(cos(angle)/2,sin(angle)/2,'k'), hold on
h1=plot(It(1,1),It(2,1),'o','LineWidth',lw,'Color',color,'MarkerFaceColor',color*sh+1-sh);
h2=plot([0 It(1,1)],[0 It(2,1)],'LineWidth',lw,'Color',color); hold off
xlabel('<{\itI_x}> (-)'), ylabel('<{\itI_y}> (-)')
xticks((-1:+1)/2), xticklabels({'-1/2','0','+1/2'})
yticks((-1:+1)/2), yticklabels({'-1/2','0','+1/2'})
set(gca,'FontSize',12), grid on
pbaspect([1 1 1])
xlim([-1 1]*0.525)
ylim([-1 1]*0.525)

% Creates an empty oject to store the handle of the spin arrow plot on the
% Bloch sphere
ArrowHandle{1}=[];

% Creates the video object
v = VideoWriter('Ivst.avi');
open(v)

% Plots the data corresponding to each frame
for k=1:K
    if k/2==round(k/2)
        ArrowHandle{1}=PlotSpin(Figure1,ArrowHandle{1},It(:,k),color,sha);
        frame = getframe(gcf);
        subplot(3,1,1)
        addpoints(px,t(k),It(1,k))
        addpoints(py,t(k),It(2,k))
        addpoints(pz,t(k),It(3,k))
        subplot(3,2,[3 5])
        h1.XData=It(1,k);
        h1.YData=It(2,k);
        h2.XData(2)=It(1,k);
        h2.YData(2)=It(2,k);
        writeVideo(v,frame)
    end
end

close(v)

%% Functions for Bloch sphere ploting

function ArrowHandle=PlotSpin(fig,ArrowHandle,ra,color,sha)
    % Spin/arrow parameters
    La = 0.15;      % Head length
    Ra = 0.06;      % Head radius
    Rab= 0.025;     % Body radius
    N  = 50;        % Arrow resolution
    
    figure(fig)
    subplot(3,2,[4 6])
    if isfield(ArrowHandle,'h1')
        delete(ArrowHandle.h1), delete(ArrowHandle.h2)
    end
    
    C(:,:,1)=color(1)*ones(2,N+1);
    C(:,:,2)=color(2)*ones(2,N+1);
    C(:,:,3)=color(3)*ones(2,N+1);
    a = acos(ra(3)/norm(ra));
    b = sign(ra(2))*acos(ra(1)/norm(ra(1:2))); 
    b(isnan(b))=0;
    M=makehgtform('scale',La,'zrotate',b,'yrotate',a,'translate',[0 0 (1-La)/La]);
    M=hgtransform('Matrix',M);
    [X,Y,Z]=cylinder([Ra/La 0],N);
    ArrowHandle.h1=surf(X,Y,Z,C,'Parent',M,'LineStyle','none','FaceAlpha',sha);
    M=makehgtform('scale',1-La,'zrotate',b,'yrotate',a);
    M=hgtransform('Matrix',M);
    [X,Y,Z]=cylinder(Rab/(1-La),N);
    ArrowHandle.h2=surf(X,Y,Z,C,'Parent',M,'LineStyle','none','FaceAlpha',sha);
end
function figure1=BlochSpereFunc(FontSize)
    % Axes/arrows parameters
    LA = 0.075;     % Head length
    RA = 0.025;     % Head radius
    shA= 1.0;       % Shading 
    R  = 1.15;      % Axes length
    N  = 50;        % Arrow resolution
    
    clf("reset")
    figure1=figure(1);
    subplot(3,2,[4 6])
    lc=[1 1 1]*0.5;
    lc2=[1 1 1]*0.2;
    lw1=1; lw2=2;
    
    % Plots the sphere
    [xs,ys,zs]=sphere(40);
    surf(xs,ys,zs,'FaceAlpha',0.1,'EdgeColor','none'), hold on
    
    % Plots the circles
    phi=linspace(0,2*pi,50);
    xc=cos(phi);
    yc=sin(phi);
    zc=xc*0;
    plot3(xc,yc,zc,'Color',lc,'LineWidth',lw1)
    plot3(yc,zc,xc,'Color',lc,'LineWidth',lw1)
    plot3(zc,xc,yc,'Color',lc,'LineWidth',lw1)
    
    % Plots the axes lines
    plot3([0 0],[0 0],[0 R-LA],'Color',lc2,'LineWidth',lw2)
    plot3([0 0],[0 R-LA],[0 0],'Color',lc2,'LineWidth',lw2)
    plot3([0 R-LA],[0 0],[0 0],'Color',lc2,'LineWidth',lw2)
    
    % Plots the axes arrow heads
    [X,Y,Z]=cylinder([RA/LA 0],N);
    C(:,:,1)=ones(2,N+1);
    C(:,:,2)=ones(2,N+1);
    C(:,:,3)=ones(2,N+1);
    M=makehgtform('scale',LA,'zrotate',0,'yrotate',0,'translate',[0 0 (R-LA)/LA]);
    M=hgtransform('Matrix',M);
    surf(X,Y,Z,C*0,'Parent',M,'LineStyle','none','FaceAlpha',shA);
    M=makehgtform('scale',LA,'zrotate',0,'yrotate',pi/2,'translate',[0 0 (R-LA)/LA]);
    M=hgtransform('Matrix',M);
    surf(X,Y,Z,C*0,'Parent',M,'LineStyle','none','FaceAlpha',shA);
    M=makehgtform('scale',LA,'zrotate',pi/2,'yrotate',pi/2,'translate',[0 0 (R-LA)/LA]);
    M=hgtransform('Matrix',M);
    surf(X,Y,Z,C*0,'Parent',M,'LineStyle','none','FaceAlpha',shA);
    
    % Labels the axes
    text(R*1.1,0,0,'{\itx}','HorizontalAlignment','center','FontSize',FontSize)
    text(0,R*1.1,0,'{\ity}','HorizontalAlignment','center','FontSize',FontSize)
    text(0,0,R*1.1,'{\itz}','HorizontalAlignment','center','FontSize',FontSize)
    
    % Sets options
    set(gca,'Xdir','reverse'), set(gca,'ydir','reverse')
    axis off
    colormap gray
    axis(gca,'tight');
    hold off
    view([-28.2,11.37])
    set(gca,'DataAspectRatio',[1 1 1]);
end


